@@ -25,7 +25,7 @@ enum book_type
 #if TAG_MAJOR_VERSION == 34
     BOOK_WAR_CHANTS = BOOK_FEN,
 #endif
-    BOOK_VAPOURS,
+    BOOK_SWAMP_SOJOURN,
     BOOK_NECROMANCY,
     BOOK_CALLINGS,
 #if TAG_MAJOR_VERSION == 34
@@ -121,6 +121,14 @@ enum book_type
     BOOK_SIEGECRAFT,
     BOOK_MALADIES,
     BOOK_CONDUCTIVITY,
+    BOOK_CONSTRUCTION,
+    BOOK_FORTRESS,
+    BOOK_TRAPS,
+    BOOK_CRYOFORMATION,
+    BOOK_GRAVE,
+    BOOK_METALWORKING,
+    BOOK_DUALITY,
+    BOOK_CONTRAPTIONS,
     NUM_BOOKS
 };
 
