@@ -9,9 +9,12 @@
 #include <string>
 
 #include "enum.h"
+#include "equipment-slot.h"
 #include "item-prop-enum.h"
 #include "object-selector-type.h"
 #include "operation-types.h"
+#include "player-equip.h"
+#include "unwind.h"
 
 const int ARMOUR_EQUIP_DELAY = 5;
 
@@ -21,35 +24,16 @@ operation_types use_an_item_menu(item_def *&target, operation_types oper,
                 function<bool ()> allowcancel = [](){ return true; });
 // Change the lambda to always_true<> when g++ 4.7 support is dropped.
 
-bool armour_prompt(const string & mesg, int *index, operation_types oper);
-
-bool takeoff_armour(int index, bool noask = false);
-
+bool oni_drunken_swing();
 bool drink(item_def* potion = nullptr);
 
 bool god_hates_brand(const int brand);
 
-bool safe_to_remove(const item_def &item, bool quiet = false);
-
-bool puton_ring(item_def &to_puton, bool allow_prompt = true,
-                bool check_for_inscriptions = true, bool noask = false);
-
 bool scroll_hostile_check(scroll_type which_scroll);
 bool scroll_has_targeter(scroll_type which_scroll);
 bool read(item_def* scroll = nullptr, dist *target=nullptr);
 
-bool remove_ring(int slot, bool announce = false, bool noask = false);
-
-bool wear_armour(int slot);
-
-bool can_wear_armour(const item_def &item, bool verbose, bool ignore_temporary);
-
-bool can_wield(const item_def *weapon, bool say_why = false,
-               bool ignore_temporary_disability = false, bool unwield = false);
-
 bool auto_wield();
-bool wield_weapon(int slot);
-bool unwield_weapon(const item_def &wpn);
 
 bool use_an_item(operation_types oper, item_def *target=nullptr);
 
@@ -64,3 +48,12 @@ bool has_drunken_brawl_targets();
 
 string item_equip_verb(const item_def& item);
 string item_unequip_verb(const item_def& item);
+
+bool handle_chain_removal(vector<item_def*>& to_remove, bool interactive);
+bool try_equip_item(item_def& item);
+bool try_unequip_item(item_def& item);
+bool warn_about_changing_gear(const vector<item_def*>& to_remove,
+                              item_def* to_equip = nullptr);
+
+void do_equipment_change(item_def* to_equip, equipment_slot equip_slot,
+                         vector<item_def*> to_remove);
