@@ -7,6 +7,7 @@
 
 #include "mon-poly.h"
 
+#include "areas.h"
 #include "artefact.h"
 #include "art-enum.h"
 #include "attitude-change.h"
@@ -33,6 +34,7 @@
 #include "state.h"
 #include "stringutil.h"
 #include "terrain.h"
+#include "transform.h"
 #include "traps.h"
 #include "xom.h"
 
@@ -67,42 +69,52 @@ void monster_drop_things(monster* mons,
         if (item == NON_ITEM || !suitable(env.item[item]))
             continue;
 
+        mons->do_unequip_effects(env.item[item], false, true);
+
+        int old_halo = mons->halo_radius();
+        int old_umbra = mons->umbra_radius();
+
         if (testbits(env.item[item].flags, ISFLAG_SUMMONED))
         {
             item_was_destroyed(env.item[item]);
             destroy_item(item);
+        }
+        else
+        {
+            if (mark_item_origins && env.item[item].defined())
+                origin_set_monster(env.item[item], mons);
+
+            env.item[item].props[DROPPER_MID_KEY].get_int() = mons->mid;
+
+            if (env.item[item].props.exists("autoinscribe"))
+            {
+                add_inscription(env.item[item],
+                    env.item[item].props["autoinscribe"].get_string());
+                env.item[item].props.erase("autoinscribe");
+            }
+
+            // If a monster is swimming, the items are ALREADY underwater.
+            if (move_item_to_grid(&item, mons->pos(), mons->swimming())
+                && player_under_penance(GOD_GOZAG)
+                // Dropping items into water/lava may have destroyed them
+                && item != NON_ITEM
+                && env.item[item].base_type == OBJ_GOLD
+                && you.see_cell(mons->pos())
+                && x_chance_in_y(env.item[item].quantity, 100)
+                && you.can_be_dazzled())
+            {
+                string msg = make_stringf("%s dazzles you with the glint of coin.",
+                    god_name(GOD_GOZAG).c_str());
+                mprf(MSGCH_GOD, GOD_GOZAG, "%s", msg.c_str());
+                blind_player(10 + random2(8), ETC_GOLD);
+            }
             mons->inv[i] = NON_ITEM;
-            continue;
         }
 
-        if (mark_item_origins && env.item[item].defined())
-            origin_set_monster(env.item[item], mons);
-
-        env.item[item].props[DROPPER_MID_KEY].get_int() = mons->mid;
-
-        if (env.item[item].props.exists("autoinscribe"))
-        {
-            add_inscription(env.item[item],
-                env.item[item].props["autoinscribe"].get_string());
-            env.item[item].props.erase("autoinscribe");
-        }
-
-        // If a monster is swimming, the items are ALREADY underwater.
-        if (move_item_to_grid(&item, mons->pos(), mons->swimming())
-            && player_under_penance(GOD_GOZAG)
-            // Dropping items into water/lava may have destroyed them
-            && item != NON_ITEM
-            && env.item[item].base_type == OBJ_GOLD
-            && you.see_cell(mons->pos())
-            && x_chance_in_y(env.item[item].quantity, 100)
-            && you.can_be_dazzled())
-        {
-            string msg = make_stringf("%s dazzles you with the glint of coin.",
-                                       god_name(GOD_GOZAG).c_str());
-            mprf(MSGCH_GOD, GOD_GOZAG, "%s", msg.c_str());
-            blind_player(10 + random2(8), ETC_GOLD);
-        }
-        mons->inv[i] = NON_ITEM;
+        int new_halo = mons->halo_radius();
+        int new_umbra = mons->umbra_radius();
+        if (old_halo != new_halo || old_umbra != new_umbra)
+            invalidate_agrid(true);
     }
 }
 
@@ -174,7 +186,7 @@ static bool _valid_morph(monster* mons, monster_type new_mclass)
     }
 
     // Determine if the monster is happy on current tile.
-    return monster_habitable_grid(new_mclass, env.grid(mons->pos()));
+    return monster_habitable_grid(new_mclass, mons->pos());
 }
 
 static bool _jiyva_slime_target(monster_type targetc)
@@ -225,7 +237,7 @@ void change_monster_type(monster* mons, monster_type targetc, bool do_seen)
     }
 
     // Inform listeners that the original monster is gone.
-    fire_monster_death_event(mons, KILL_MISC, true);
+    fire_monster_death_event(mons, KILL_NON_ACTOR, true);
 
     // the actual polymorphing:
     auto flags =
@@ -302,18 +314,18 @@ void change_monster_type(monster* mons, monster_type targetc, bool do_seen)
     if (!mons->props.exists(ORIG_HD_KEY))
         mons->props[ORIG_HD_KEY] = mons->get_experience_level();
 
-    mon_enchant abj       = mons->get_ench(ENCH_ABJ);
-    mon_enchant fabj      = mons->get_ench(ENCH_FAKE_ABJURATION);
+    mon_enchant timer     = mons->get_ench(ENCH_SUMMON_TIMER);
     mon_enchant charm     = mons->get_ench(ENCH_CHARM);
     mon_enchant shifter   = mons->get_ench(ENCH_GLOWING_SHAPESHIFTER,
                                            ENCH_SHAPESHIFTER);
     mon_enchant summon    = mons->get_ench(ENCH_SUMMON);
     mon_enchant tp        = mons->get_ench(ENCH_TP);
-    mon_enchant vines     = mons->get_ench(ENCH_AWAKEN_VINES);
     mon_enchant forest    = mons->get_ench(ENCH_AWAKEN_FOREST);
     mon_enchant hexed     = mons->get_ench(ENCH_HEXED);
     mon_enchant insanity  = mons->get_ench(ENCH_FRENZIED);
     mon_enchant vengeance = mons->get_ench(ENCH_VENGEANCE_TARGET);
+    mon_enchant tempered  = mons->get_ench(ENCH_TEMPERED);
+    mon_enchant thrall    = mons->get_ench(ENCH_VAMPIRE_THRALL);
 
     mons->number       = 0;
 
@@ -345,17 +357,17 @@ void change_monster_type(monster* mons, monster_type targetc, bool do_seen)
     mons->god = (mons->is_priest() && old_god == GOD_NO_GOD) ? GOD_NAMELESS
                                                              : old_god;
 
-    mons->add_ench(abj);
-    mons->add_ench(fabj);
+    mons->add_ench(timer);
     mons->add_ench(charm);
     mons->add_ench(shifter);
     mons->add_ench(summon);
     mons->add_ench(tp);
-    mons->add_ench(vines);
     mons->add_ench(forest);
     mons->add_ench(hexed);
     mons->add_ench(insanity);
     mons->add_ench(vengeance);
+    mons->add_ench(tempered);
+    mons->add_ench(thrall);
 
     mons->ench_countdown = old_ench_countdown;
 
@@ -423,23 +435,8 @@ static bool _habitat_matches(bool orig_flies, habitat_type orig_hab,
         return false;
 
     const habitat_type new_hab = mons_habitat_type(new_type, new_type, false);
-    switch (orig_hab)
-    {
-        case HT_AMPHIBIOUS:
-        case HT_AMPHIBIOUS_LAVA:
-            return new_hab == orig_hab;
-        case HT_WATER:
-            return new_hab == orig_hab || new_hab == HT_AMPHIBIOUS;
-        case HT_LAVA:
-            return new_hab == orig_hab || new_hab == HT_AMPHIBIOUS_LAVA;
-        case HT_LAND:
-            return new_hab == orig_hab
-                || new_hab == HT_AMPHIBIOUS
-                || new_hab == HT_AMPHIBIOUS_LAVA;
-        case NUM_HABITATS:
-            break;
-    }
-    return false; // should never happen
+
+    return (new_hab & orig_hab) == orig_hab;
 }
 
 static int _goal_hd(int orig_hd, poly_power_type ppt)
@@ -714,7 +711,7 @@ void slimify_monster(monster* mon)
     const monster_type target = _slime_target(*mon);
 
     // Bail out if jellies can't live here.
-    if (!monster_habitable_grid(target, env.grid(mon->pos())))
+    if (!monster_habitable_grid(target, mon->pos()))
     {
         simple_monster_message(*mon, " quivers momentarily.");
         return;
@@ -774,9 +771,17 @@ void seen_monster(monster* mons)
         take_note(Note(NOTE_SEEN_MONSTER, mons->type, 0, name));
     }
 
-    if (player_equip_unrand(UNRAND_WYRMBANE))
+    if (you.unrand_equipped(UNRAND_WYRMBANE))
     {
-        const item_def *wyrmbane = you.weapon();
+        const item_def *wyrmbane = nullptr;
+        const item_def *wpn = you.weapon();
+        const item_def *offhand_wpn = you.offhand_weapon();
+
+        if (wpn && wpn->unrand_idx == UNRAND_WYRMBANE)
+            wyrmbane = wpn;
+        else if (offhand_wpn && offhand_wpn->unrand_idx == UNRAND_WYRMBANE)
+            wyrmbane = offhand_wpn;
+
         if (wyrmbane && mons->dragon_level() > wyrmbane->plus)
             mpr("<green>Wyrmbane glows as a worthy foe approaches.</green>");
     }
@@ -796,4 +801,7 @@ void seen_monster(monster* mons)
 
     if (mons_offers_beogh_conversion(*mons))
         env.level_state |= LSTATE_BEOGH;
+
+    if (you.form == transformation::sphinx)
+        sphinx_notice_riddle_target(mons);
 }
