@@ -21,6 +21,7 @@
 #endif
 
 #include "artefact.h"
+#include "act-iter.h"
 #include "art-enum.h"
 #include "beam.h"
 #include "chardump.h"
@@ -41,11 +42,13 @@
 #include "item-prop.h"
 #include "items.h"
 #include "libutil.h"
+#include "melee-attack.h"
 #include "message.h"
 #include "mgen-data.h"
 #include "mon-death.h"
 #include "mon-place.h"
 #include "mon-speak.h"
+#include "mon-tentacle.h"
 #include "mon-util.h"
 #include "mutation.h"
 #include "nearby-danger.h"
@@ -63,10 +66,12 @@
 #include "spl-clouds.h"
 #include "spl-damage.h"
 #include "spl-goditem.h"
+#include "spl-monench.h"
 #include "spl-selfench.h"
 #include "state.h"
 #include "stringutil.h"
 #include "teleport.h"
+#include "terrain.h"
 #include "transform.h"
 #include "tutorial.h"
 #include "view.h"
@@ -157,7 +162,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
             canned_msg(MSG_YOU_RESIST);
         else if (hurted > original && doEffects)
         {
-            mpr("You feel a terrible chill!");
+            mpr("The cold chills you terribly!");
             xom_is_stimulated(200);
         }
         break;
@@ -242,7 +247,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
             canned_msg(MSG_YOU_PARTIALLY_RESIST);
         else if (hurted > original && doEffects)
         {
-            mpr("You feel a painful chill!");
+            mpr("The ice freezes you terribly!");
             xom_is_stimulated(200);
         }
         break;
@@ -293,19 +298,6 @@ int check_your_resists(int hurted, beam_type flavour, string source,
             you.strip_willpower(beam->agent(), random_range(8, 14));
         break;
 
-    case BEAM_CRYSTALLIZING:
-        if (doEffects)
-        {
-            if (x_chance_in_y(2, 3)) {
-                if (!you.duration[DUR_VITRIFIED])
-                    mpr("Your body becomes as fragile as glass!");
-                else
-                    mpr("You feel your fragility will last longer.");
-                you.increase_duration(DUR_VITRIFIED, random_range(8, 18), 50);
-            }
-        }
-        break;
-
     case BEAM_UMBRAL_TORCHLIGHT:
         if (you.holiness() & ~(MH_NATURAL | MH_DEMONIC | MH_HOLY)
             || beam->agent(true)->is_player())
@@ -322,6 +314,24 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         }
         break;
 
+    case BEAM_SEISMIC:
+        if (you.airborne())
+            hurted = hurted / 3;
+        break;
+
+    case BEAM_BOLAS:
+        if (doEffects)
+        {
+            if (you.is_insubstantial() || you.is_amorphous())
+                mpr("The bolas passes through you!");
+            else
+            {
+                you.set_duration(DUR_NO_MOMENTUM, random_range(4, 8), 0,
+                "The bolas wraps around you and binds you in place!");
+            }
+        }
+        break;
+
     default:
         break;
     }                           // end switch
@@ -366,6 +376,15 @@ void expose_player_to_element(beam_type flavour, int strength, bool slow_cold_bl
         mprf(MSGCH_WARN, "The flames go out!");
         end_sticky_flame_player();
     }
+
+    if (you.form == transformation::aqua
+        && (flavour == BEAM_COLD || flavour == BEAM_ICE)
+        && coinflip())
+    {
+        if (!you.duration[DUR_FROZEN])
+            mpr("Your body starts to freeze solid!");
+        you.increase_duration(DUR_FROZEN, random_range(5, 10), 50);
+    }
 }
 
 void lose_level()
@@ -570,7 +589,7 @@ static void _maybe_spawn_rats(int dam, kill_method_type death_type)
 {
     if (dam <= 0
         || death_type == KILLED_BY_POISON
-        || !player_equip_unrand(UNRAND_RATSKIN_CLOAK))
+        || !you.unrand_equipped(UNRAND_RATSKIN_CLOAK))
     {
         return;
     }
@@ -593,10 +612,10 @@ static void _maybe_spawn_rats(int dam, kill_method_type death_type)
         return;
 
     mgen_data mg(mon, BEH_FRIENDLY, you.pos(), MHITYOU);
+    mg.set_summoned(&you, SPELL_NO_SPELL, summ_dur(3), false);
     mg.flags |= MG_FORCE_BEH; // don't mention how much it hates you before it appears
     if (monster *m = create_monster(mg))
     {
-        m->add_ench(mon_enchant(ENCH_FAKE_ABJURATION, 3));
         mprf("%s scurries out from under your cloak.", m->name(DESC_A).c_str());
         // We should return early in the case of no_love or no_allies,
         // so this is more a sanity check.
@@ -619,6 +638,65 @@ static void _maybe_summon_demonic_guardian(int dam, kill_method_type death_type)
     }
 }
 
+// The time-warped blood mutation grants haste to
+// your allies when you're brought below half health.
+void _maybe_blood_hastes_allies()
+{
+    if (you.hp * 2 > you.hp_max
+        || !you.has_mutation(MUT_TIME_WARPED_BLOOD)
+        || you.duration[DUR_TIME_WARPED_BLOOD_COOLDOWN])
+    {
+        return;
+    }
+
+    vector<monster*> targetable;
+    int target_count = you.get_mutation_level(MUT_TIME_WARPED_BLOOD) * 2;
+    int affected = 0;
+    int time = random_range(20, 30);
+
+    you.duration[DUR_TIME_WARPED_BLOOD_COOLDOWN] = 1;
+
+    for (monster_near_iterator mi(you.pos(), LOS_NO_TRANS); mi; ++mi)
+    {
+        // Try to look for valid allies that aren't already hasted,
+        // and which would properly function when given haste.
+        if (mi->alive() && mons_attitude(**mi) == ATT_FRIENDLY
+            && !mi->berserk_or_frenzied() && you.can_see(**mi)
+            && !mi->has_ench(ENCH_HASTE)
+            && !mi->is_peripheral())
+        {
+            targetable.emplace_back(*mi);
+        }
+    }
+
+    if (targetable.empty())
+    {
+       mpr("Your atemporal blood churns to no real effect.");
+       return;
+    }
+
+    // Affect the highest HD allies you have.
+    shuffle_array(targetable);
+    sort(targetable.begin(), targetable.end(),
+         [](const monster* a, const monster* b)
+         {return a->get_hit_dice() > b->get_hit_dice();});
+
+    for (monster *application: targetable)
+    {
+        if (affected < target_count)
+        {
+             flash_tile(application->pos(), BLUE, 0);
+             animation_delay(15, true);
+             application->add_ench(mon_enchant(ENCH_HASTE, 0, &you,
+                                   time * BASELINE_DELAY));
+             affected++;
+        }
+    }
+
+    if (affected > 0)
+        mpr("The spilling of your atemporal blood hastes your allies!");
+}
+
 static void _maybe_spawn_monsters(int dam, kill_method_type death_type,
                                   mid_t death_source)
 {
@@ -655,8 +733,8 @@ static void _maybe_spawn_monsters(int dam, kill_method_type death_type,
         for (int i = 0; i < how_many; ++i)
         {
             const int mindex = damager->alive() ? damager->mindex() : MHITNOT;
-            mgen_data mg(mon, BEH_FRIENDLY, you.pos(), mindex);
-            mg.set_summoned(&you, 2, 0, you.religion);
+            mgen_data mg(mon, BEH_FRIENDLY, you.pos(), mindex, MG_NONE, you.religion);
+            mg.set_summoned(&you, 0, summ_dur(2));
 
             if (create_monster(mg))
                 count_created++;
@@ -726,13 +804,131 @@ static void _maybe_fog(int dam)
     }
 }
 
-static void _deteriorate(int dam)
+static void _maybe_splash_water(int dam)
 {
-    if (x_chance_in_y(you.get_mutation_level(MUT_DETERIORATION), 4)
-        && dam > you.hp_max / 10)
+    if (you.form != transformation::aqua)
+        return;
+
+    const int percent = dam * 100 / you.hp_max;
+
+    if (percent < 10 && !one_chance_in(percent))
+        return;
+
+    // Assume the player can fill 25 tiles will all their hp (with some randomisation).
+    int water = div_rand_round(percent, 2);
+
+    if (water == 0)
+        return;
+
+    vector<coord_def> spots;
+    for (distance_iterator di(you.pos(), true, false, 3); di; ++di)
     {
-        mprf(MSGCH_WARN, "Your body deteriorates!");
-        lose_stat(STAT_RANDOM, 1);
+        if (you.see_cell_no_trans(*di)
+            && feat_has_dry_floor(env.grid(*di))
+            && !feat_is_critical(env.grid(*di)))
+        {
+            spots.push_back(*di);
+        }
+    }
+
+    // spots is neatly ordered from player->outwards. random_spots is completely
+    // randomized. We mostly pick from spots, with a smaller chance to pick from
+    // random spots, so that the splash tends to build outward from the player,
+    // but slightly unevenly.
+    vector<coord_def> random_spots = spots;
+    shuffle_array(random_spots);
+
+    water = min(water, (int)spots.size());
+
+    if (water == 0)
+        return;
+
+    mpr("You splash onto the ground.");
+    for (int i = 0; i < water; ++i)
+    {
+        const coord_def pos = one_chance_in(3) ? random_spots[i] : spots[i];
+        temp_change_terrain(pos, DNGN_SHALLOW_WATER, random_range(80, 110),
+                            TERRAIN_CHANGE_AQUA_FORM, MID_PLAYER);
+    }
+}
+
+static void _maybe_hive_swarm()
+{
+    if (you.form != transformation::hive
+        || you.allies_forbidden()
+        || you.hp * 2 > you.hp_max
+        || you.duration[DUR_HIVE_COOLDOWN])
+    {
+        return;
+    }
+
+    mgen_data mg(MONS_KILLER_BEE, BEH_FRIENDLY, you.pos(), MHITYOU, MG_FORCE_BEH | MG_AUTOFOE);
+    mg.set_summoned(&you, MON_SUMM_HIVE, random_range(12, 18) * BASELINE_DELAY, false).set_range(1, 3);
+    mg.hd = 5;
+
+    const int num = div_rand_round(get_form()->get_effect_size(), 10);
+    bool made_mon = false;
+    for (int i = 0; i < num; ++i)
+    {
+        if (monster *swarmer = create_monster(mg))
+        {
+            made_mon = true;
+            swarmer->add_ench(ENCH_BERSERK);
+            swarmer->add_ench(ENCH_CONCENTRATE_VENOM);
+        }
+    }
+
+    if (made_mon)
+    {
+        mpr("Angry insects swarm out of your body to defend their hive!");
+        you.duration[DUR_HIVE_COOLDOWN] = 1;
+    }
+}
+
+static void _maybe_medusa_lithotoxin()
+{
+    if (you.form != transformation::medusa
+        || you.hp * 10 > you.hp_max * 6
+        || you.duration[DUR_MEDUSA_COOLDOWN])
+    {
+        return;
+    }
+
+    vector<monster*> targs;
+    for (radius_iterator ri(you.pos(), 3, C_SQUARE, LOS_NO_TRANS, true); ri; ++ri)
+        if (monster* mon = monster_at(*ri))
+            if (!mon->wont_attack() && mon->has_ench(ENCH_POISON))
+                targs.push_back(mon);
+
+    if (targs.empty())
+        return;
+
+    draw_ring_animation(you.pos(), 3, LIGHTGREY, 0U, true, 25);
+    mprf("Your pain echoes through the poison around you!");
+    for (monster* targ : targs)
+    {
+        if (x_chance_in_y(get_form()->get_effect_chance(), 100))
+            targ->petrify(&you);
+        else
+            simple_monster_message(*targ, " resists.");
+    }
+
+    you.duration[DUR_MEDUSA_COOLDOWN] = 1;
+}
+
+static void _handle_poor_constitution(int dam)
+{
+    const int level = you.get_mutation_level(MUT_POOR_CONSTITUTION);
+
+    if (level == 0)
+        return;
+
+    if (dam > you.hp_max / 15 && one_chance_in(level == 1 ? 9 : 6))
+    {
+        you.weaken(nullptr, 20);
+
+        if (level == 2 && one_chance_in(2))
+            you.slow_down(nullptr, random_range(8, 15));
     }
 }
 
@@ -748,7 +944,7 @@ static void _maybe_corrode()
 {
     int corrosion_sources = you.scan_artefacts(ARTP_CORRODE);
     if (x_chance_in_y(corrosion_chance(corrosion_sources), 100))
-        you.corrode_equipment("Your corrosive artefact");
+        you.corrode(nullptr, "Your corrosive artefact");
 }
 
 /**
@@ -761,6 +957,15 @@ static void _maybe_slow()
         slow_player(10 + random2(5));
 }
 
+/**
+ * Maybe silence the player after taking damage if they're wearing *Silence.
+ **/
+static void _maybe_silence()
+{
+    int silence_sources = you.scan_artefacts(ARTP_SILENCE);
+    if (x_chance_in_y(silence_sources, 100))
+        silence_player(4 + random2(7));
+}
 /**
  * Maybe disable scrolls after taking damage if the player has MUT_READ_SAFETY.
  **/
@@ -985,9 +1190,12 @@ static void _print_endgame_messages(scorefile_entry &se)
  *  @param aux what did they do it with?
  *  @param see_source whether the attacker was visible to you
  *  @param death_source_name the attacker's name if it is already dead.
+ *  @param skip_multipliers Whether to ignore harm/vitrify/etc.
+ *  @param skip_awaken Whether this damage will skip waking a sleeping player.
  */
 void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
-          bool see_source, const char *death_source_name)
+          bool see_source, const char *death_source_name, bool skip_multipliers,
+          bool skip_awaken)
 {
     ASSERT(!crawl_state.game_is_arena());
     if (you.duration[DUR_TIME_STEP])
@@ -1007,7 +1215,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
     }
 
     // Multiply damage if Harm or Vitrify is in play. (Poison is multiplied earlier.)
-    if (dam != INSTANT_DEATH && death_type != KILLED_BY_POISON)
+    if (dam != INSTANT_DEATH && death_type != KILLED_BY_POISON && !skip_multipliers)
     {
         dam = _apply_extra_harm(dam, source);
 
@@ -1038,8 +1246,8 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
     interrupt_activity(activity_interrupt::hp_loss, &hpl);
 
     // Don't wake the player with fatal or poison damage.
-    if (dam > 0 && dam < you.hp && death_type != KILLED_BY_POISON)
-        you.check_awaken(500);
+    if (dam > 0 && dam < you.hp && death_type != KILLED_BY_POISON && !skip_awaken)
+        you.wake_up();
 
     const bool non_death = death_type == KILLED_BY_QUITTING
                         || death_type == KILLED_BY_WINNING
@@ -1062,7 +1270,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
             return;
         // the dreamshard necklace protects from any fatal blow or death source
         // that death's door would protect from.
-        else if (player_equip_unrand(UNRAND_DREAMSHARD_NECKLACE)
+        else if (you.unrand_equipped(UNRAND_DREAMSHARD_NECKLACE)
                  && dam >= you.hp)
         {
             dreamshard_shatter();
@@ -1088,7 +1296,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
             // Wake players who took fatal damage exactly equal to current HP,
             // but had it reduced below fatal threshold by spirit shield.
             if (dam < you.hp)
-                you.check_awaken(500);
+                you.wake_up();
 
             if (dam <= 0 && you.hp > 0)
                 return;
@@ -1099,7 +1307,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
             simple_god_message(" protects you from harm!");
             // Ensure divine intervention wakes sleeping players. Necessary
             // because we otherwise don't wake players who take fatal damage.
-            you.check_awaken(500);
+            you.wake_up();
             return;
         }
 
@@ -1141,15 +1349,19 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
             take_note(Note(NOTE_HP_CHANGE, you.hp, you.hp_max,
                            damage_desc.c_str()));
 
-            _deteriorate(dam);
+            _handle_poor_constitution(dam);
             _maybe_ru_retribution(dam, source);
             _maybe_inflict_anguish(dam, source);
             _maybe_spawn_monsters(dam, death_type, source);
             _maybe_spawn_rats(dam, death_type);
             _maybe_summon_demonic_guardian(dam, death_type);
             _maybe_fog(dam);
+            _maybe_blood_hastes_allies();
             _powered_by_pain(dam);
             makhleb_celebrant_bloodrite();
+            _maybe_splash_water(dam);
+            _maybe_hive_swarm();
+            _maybe_medusa_lithotoxin();
             if (sanguine_armour_valid())
                 activate_sanguine_armour();
             refresh_meek_bonus();
@@ -1157,6 +1369,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
             {
                 _maybe_corrode();
                 _maybe_slow();
+                _maybe_silence();
                 _maybe_disable_scrolls();
                 _maybe_disable_potions();
             }
