@@ -38,7 +38,6 @@
 #include "god-wrath.h"
 #include "hints.h"
 #include "items.h"
-#include "item-name.h" // item_type_known
 #include "item-prop.h" // get_weapon_brand
 #include "libutil.h"
 #include "macro.h"
@@ -75,6 +74,7 @@
  #include "tileview.h"
 #endif
 #include "tiles-build-specific.h"
+#include "transform.h"
 #include "traps.h"
 #include "travel.h"
 #include "unicode.h"
@@ -84,7 +84,7 @@
 #include "xom.h"
 
 static layers_type _layers = LAYERS_ALL;
-static layers_type _layers_saved = LAYERS_NONE;
+static layers_type _layers_saved = Layer::None;
 
 crawl_view_geometry crawl_view;
 
@@ -269,7 +269,7 @@ static bool _is_mon_equipment_worth_listing(const monster_info &mi)
 static bool _does_core_name_include_inventory(const monster *mon)
 {
     return mon->type == MONS_DANCING_WEAPON || mon->type == MONS_SPECTRAL_WEAPON
-           || mon->type == MONS_ANIMATED_ARMOUR;
+           || mon->type == MONS_ARMOUR_ECHO;
 }
 
 /// Return a warning for the player about newly-seen monsters, as appropriate.
@@ -428,8 +428,12 @@ static string _describe_monsters_from_species(const vector<details> &species)
         [] (const details &det)
         {
             string name = det.name;
-            if (mons_is_unique(det.mon->type) || mons_is_specially_named(det.mon->type))
+            if (mons_is_unique(det.mon->type)
+                || mons_is_specially_named(det.mon->type)
+                || !you.can_see(*det.mon))
+            {
                 return name;
+            }
             else if (det.count > 1 && det.genus)
             {
                 auto genus = mons_genus(det.mon->type);
@@ -484,6 +488,11 @@ static void _handle_comes_into_view(const vector<string> &msgs,
 /// If the player has the shout mutation, maybe shout at newly-seen monsters.
 static void _maybe_trigger_shoutitis(const vector<monster*> monsters)
 {
+    if (you.form == transformation::maw)
+        for (const monster* mon : monsters)
+            if (maw_growl_check(mon))
+                return;
+
     if (!you.get_mutation_level(MUT_SCREAM))
         return;
 
@@ -522,8 +531,7 @@ static void _maybe_gozag_incite(vector<monster*> monsters)
         // XXX: some of this is probably redundant with interrupt_activity
         if (mon->wont_attack()
             || mon->is_stationary()
-            || mons_is_object(mon->type)
-            || mons_is_tentacle_or_tentacle_segment(mon->type))
+            || mon->is_peripheral())
         {
             continue;
         }
@@ -678,7 +686,7 @@ static colour_t _feat_default_map_colour(dungeon_feature_type feat)
 // Returns true if it succeeded.
 bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
                    bool force, bool deterministic, bool full_info,
-                   bool range_falloff, coord_def origin)
+                   bool range_falloff, coord_def origin, bool respect_no_automap)
 {
     if (!force && !is_map_persistent())
     {
@@ -707,6 +715,10 @@ bool magic_mapping(int map_radius, int proportion, bool suppress_msg,
          ri; ++ri)
     {
         coord_def pos = *ri;
+
+        if (respect_no_automap && env.pgrid(pos) & FPROP_NO_AUTOMAP)
+            continue;
+
         if (range_falloff)
         {
             int threshold = proportion;
@@ -921,9 +933,23 @@ string screenshot()
     return ss.str();
 }
 
-int viewmap_flash_colour()
+colour_t viewmap_flash_colour()
 {
-    return _layers & LAYERS_ALL && you.berserk() ? RED : BLACK;
+    // This only shows the fullscreen colour when we're showing all layers,
+    // and hides it for parseability's sake when toggling individual layers.
+    if ((_layers & LAYERS_ALL) != LAYERS_ALL)
+        return BLACK;
+
+    if (you.berserk())
+        return RED;
+    else if (you.paralysed())
+        return LIGHTBLUE;
+    else if (you.petrified())
+        return LIGHTGRAY;
+    else if (you.duration[DUR_VEXED])
+        return MAGENTA;
+
+    return BLACK;
 }
 
 // Updates one square of the view area. Should only be called for square
@@ -1339,11 +1365,6 @@ static animation *animations[NUM_ANIMATIONS] = {
 
 void run_animation(animation_type anim, use_animation_type type, bool cleanup)
 {
-#ifdef USE_TILE_WEB
-    // XXX this doesn't work in webtiles yet
-    if (is_tiles())
-        return;
-#endif
     if (Options.use_animations & type)
     {
         animation *a = animations[anim];
@@ -1480,7 +1501,7 @@ void viewwindow(bool show_updates, bool tiles_only, animation *a, view_renderer
             UNUSED(tiles_only);
 #endif
 #ifdef USE_TILE
-            tiles.set_need_redraw(you.running ? Options.tile_runrest_rate : 0);
+            tiles.set_need_redraw();
             tiles.load_dungeon(vbuf, crawl_view.vgrdc);
             tiles.update_tabs();
 #endif
@@ -1526,7 +1547,7 @@ void view_add_glyph_overlay(const coord_def &gc, cglyph_t glyph)
 }
 
 // Simple helper function to reduce duplication with repeatedly used animation code
-void flash_tile(coord_def p, colour_t colour, int delay)
+void flash_tile(coord_def p, colour_t colour, int delay, tileidx_t tile)
 {
     if (!(Options.use_animations & UA_BEAM))
         return;
@@ -1535,7 +1556,14 @@ void flash_tile(coord_def p, colour_t colour, int delay)
         return;
 
 #ifdef USE_TILE
-        view_add_tile_overlay(p, tileidx_zap(colour));
+        // Use a bolt tile if one is specified. Otherwise, just use the default
+        // for the colour provied.
+        if (tile > 0)
+            view_add_tile_overlay(p, vary_bolt_tile(tile));
+        else
+            view_add_tile_overlay(p, tileidx_zap(colour));
+#else
+        UNUSED(tile);
 #endif
         view_add_glyph_overlay(p, {dchar_glyph(DCHAR_FIRED_ZAP),
                                    static_cast<unsigned short>(colour)});
@@ -1552,6 +1580,32 @@ void view_clear_overlays()
     glyph_overlays.clear();
 }
 
+void draw_ring_animation(const coord_def& center, int radius, colour_t colour,
+                         colour_t colour_alt, bool outward, int delay)
+{
+    const int start = outward ? 0 : radius;
+    const int end = outward ? radius : 0;
+    const int step = outward ? 1 : -1;
+
+    for (int i = start; i != end; i += step)
+    {
+        for (distance_iterator di(center, false, false, i); di; ++di)
+        {
+            if (grid_distance(center, *di) == i && !feat_is_solid(env.grid(*di))
+                && you.see_cell_no_trans(*di))
+            {
+                colour_t draw_colour = colour_alt != BLACK ? coinflip() ? colour
+                                                                        : colour_alt
+                                                           : colour;
+                flash_tile(*di, draw_colour, 0);
+            }
+        }
+
+        animation_delay(delay, true);
+        view_clear_overlays();
+    }
+}
+
 /**
  * Comparison function for coord_defs that orders coords based on the ordering
  * used by rectangle_iterator.
@@ -1590,12 +1644,7 @@ static void add_overlays(const coord_def& gc, screen_cell_t* cell)
            && tile_overlays[tile_overlay_i].gc == gc)
     {
         const auto &overlay = tile_overlays[tile_overlay_i];
-        if (cell->tile.num_dngn_overlay == 0
-            || cell->tile.dngn_overlay[cell->tile.num_dngn_overlay - 1]
-                                            != static_cast<int>(overlay.tile))
-        {
-            cell->tile.dngn_overlay[cell->tile.num_dngn_overlay++] = overlay.tile;
-        }
+        cell->tile.add_overlay(overlay.tile);
         tile_overlay_i++;
     }
 #endif
@@ -1670,7 +1719,7 @@ void draw_cell(screen_cell_t *cell, const coord_def &gc,
     else if (!crawl_view.in_los_bounds_g(gc))
         _draw_outside_los(cell, gc, coord_def());
     else if (gc == you.pos() && you.on_current_level
-             && _layers & LAYER_PLAYER
+             && _layers & Layer::PLAYER
              && !crawl_state.game_is_arena()
              && !crawl_state.arena_suspended)
     {
@@ -1777,7 +1826,7 @@ void draw_cell(screen_cell_t *cell, const coord_def &gc,
     if ((_layers != LAYERS_ALL || Options.always_show_exclusions)
         && you.on_current_level
         && map_bounds(gc)
-        && (_layers == LAYERS_NONE
+        && (_layers == Layer::None
             || gc != you.pos()
                && (env.map_knowledge(gc).monster() == MONS_NO_MONSTER
                    || !you.see_cell(gc)))
@@ -1800,8 +1849,8 @@ static void _config_layers_menu()
     bool exit = false;
 
     _layers = _layers_saved;
-    crawl_state.viewport_weapons    = !!(_layers & LAYER_MONSTER_WEAPONS);
-    crawl_state.viewport_monster_hp = !!(_layers & LAYER_MONSTER_HEALTH);
+    crawl_state.viewport_weapons    = !!(_layers & Layer::MONSTER_WEAPONS);
+    crawl_state.viewport_monster_hp = !!(_layers & Layer::MONSTER_HEALTH);
 
     msgwin_set_temporary(true);
     while (!exit)
@@ -1819,20 +1868,20 @@ static void _config_layers_menu()
                            "<%s>monster (h)ealth</%s>"
 #endif
                            ,
-           _layers & LAYER_MONSTERS        ? "lightgrey" : "darkgrey",
-           _layers & LAYER_MONSTERS        ? "lightgrey" : "darkgrey",
-           _layers & LAYER_PLAYER          ? "lightgrey" : "darkgrey",
-           _layers & LAYER_PLAYER          ? "lightgrey" : "darkgrey",
-           _layers & LAYER_ITEMS           ? "lightgrey" : "darkgrey",
-           _layers & LAYER_ITEMS           ? "lightgrey" : "darkgrey",
-           _layers & LAYER_CLOUDS          ? "lightgrey" : "darkgrey",
-           _layers & LAYER_CLOUDS          ? "lightgrey" : "darkgrey"
+           _layers & Layer::MONSTERS        ? "lightgrey" : "darkgrey",
+           _layers & Layer::MONSTERS        ? "lightgrey" : "darkgrey",
+           _layers & Layer::PLAYER          ? "lightgrey" : "darkgrey",
+           _layers & Layer::PLAYER          ? "lightgrey" : "darkgrey",
+           _layers & Layer::ITEMS           ? "lightgrey" : "darkgrey",
+           _layers & Layer::ITEMS           ? "lightgrey" : "darkgrey",
+           _layers & Layer::CLOUDS          ? "lightgrey" : "darkgrey",
+           _layers & Layer::CLOUDS          ? "lightgrey" : "darkgrey"
 #ifndef USE_TILE_LOCAL
            ,
-           _layers & LAYER_MONSTER_WEAPONS ? "lightgrey" : "darkgrey",
-           _layers & LAYER_MONSTER_WEAPONS ? "lightgrey" : "darkgrey",
-           _layers & LAYER_MONSTER_HEALTH  ? "lightgrey" : "darkgrey",
-           _layers & LAYER_MONSTER_HEALTH  ? "lightgrey" : "darkgrey"
+           _layers & Layer::MONSTER_WEAPONS ? "lightgrey" : "darkgrey",
+           _layers & Layer::MONSTER_WEAPONS ? "lightgrey" : "darkgrey",
+           _layers & Layer::MONSTER_HEALTH  ? "lightgrey" : "darkgrey",
+           _layers & Layer::MONSTER_HEALTH  ? "lightgrey" : "darkgrey"
 #endif
         );
         mprf(MSGCH_PROMPT, "Press 'a' to toggle all layers. "
@@ -1840,28 +1889,28 @@ static void _config_layers_menu()
 
         switch (get_ch())
         {
-        case 'm': _layers_saved = _layers ^= LAYER_MONSTERS;        break;
-        case 'p': _layers_saved = _layers ^= LAYER_PLAYER;          break;
-        case 'i': _layers_saved = _layers ^= LAYER_ITEMS;           break;
-        case 'c': _layers_saved = _layers ^= LAYER_CLOUDS;          break;
+        case 'm': _layers_saved = _layers ^= Layer::MONSTERS;        break;
+        case 'p': _layers_saved = _layers ^= Layer::PLAYER;          break;
+        case 'i': _layers_saved = _layers ^= Layer::ITEMS;           break;
+        case 'c': _layers_saved = _layers ^= Layer::CLOUDS;          break;
 #ifndef USE_TILE_LOCAL
-        case 'w': _layers_saved = _layers ^= LAYER_MONSTER_WEAPONS;
-                  if (_layers & LAYER_MONSTER_WEAPONS)
-                      _layers_saved = _layers |= LAYER_MONSTERS;
+        case 'w': _layers_saved = _layers ^= Layer::MONSTER_WEAPONS;
+                  if (_layers & Layer::MONSTER_WEAPONS)
+                      _layers_saved = _layers |= Layer::MONSTERS;
                   break;
-        case 'h': _layers_saved = _layers ^= LAYER_MONSTER_HEALTH;
-                  if (_layers & LAYER_MONSTER_HEALTH)
-                      _layers_saved = _layers |= LAYER_MONSTERS;
+        case 'h': _layers_saved = _layers ^= Layer::MONSTER_HEALTH;
+                  if (_layers & Layer::MONSTER_HEALTH)
+                      _layers_saved = _layers |= Layer::MONSTERS;
                   break;
 #endif
         case 'a': if (_layers)
-                      _layers_saved = _layers = LAYERS_NONE;
+                      _layers_saved = _layers = Layer::None;
                   else
                   {
 #ifndef USE_TILE_LOCAL
                       _layers_saved = _layers = LAYERS_ALL
-                                      | LAYER_MONSTER_WEAPONS
-                                      | LAYER_MONSTER_HEALTH;
+                                      | Layer::MONSTER_WEAPONS
+                                      | Layer::MONSTER_HEALTH;
 #else
                       _layers_saved = _layers = LAYERS_ALL;
 #endif
@@ -1869,14 +1918,14 @@ static void _config_layers_menu()
                   break;
         default:
             _layers = LAYERS_ALL;
-            crawl_state.viewport_weapons    = !!(_layers & LAYER_MONSTER_WEAPONS);
-            crawl_state.viewport_monster_hp = !!(_layers & LAYER_MONSTER_HEALTH);
+            crawl_state.viewport_weapons    = !!(_layers & Layer::MONSTER_WEAPONS);
+            crawl_state.viewport_monster_hp = !!(_layers & Layer::MONSTER_HEALTH);
             exit = true;
             break;
         }
 
-        crawl_state.viewport_weapons    = !!(_layers & LAYER_MONSTER_WEAPONS);
-        crawl_state.viewport_monster_hp = !!(_layers & LAYER_MONSTER_HEALTH);
+        crawl_state.viewport_weapons    = !!(_layers & Layer::MONSTER_WEAPONS);
+        crawl_state.viewport_monster_hp = !!(_layers & Layer::MONSTER_HEALTH);
 
         msgwin_clear_temporary();
     }
@@ -1899,8 +1948,8 @@ void reset_show_terrain()
         mprf(MSGCH_PROMPT, "Restoring view layers.");
 
     _layers = LAYERS_ALL;
-    crawl_state.viewport_weapons    = !!(_layers & LAYER_MONSTER_WEAPONS);
-    crawl_state.viewport_monster_hp = !!(_layers & LAYER_MONSTER_HEALTH);
+    crawl_state.viewport_weapons    = !!(_layers & Layer::MONSTER_WEAPONS);
+    crawl_state.viewport_monster_hp = !!(_layers & Layer::MONSTER_HEALTH);
 }
 
 ////////////////////////////////////////////////////////////////////////////
